import win32serviceutil

def is_service_running (service, machine=''):
	if win32serviceutil.QueryServiceStatus(service, machine)[1] == 4:
		return True
	else:
		return False
	
def service_control(service, action, machine=''):
	if action == 'stop':
		try:
			win32serviceutil.StopService(service, machine)
			return True
		except:
			return False
	elif action == 'start':
		try:
			win32serviceutil.StartService(service, machine)
			return True
		except:
			return False             
	elif action == 'restart':
		try:
			win32serviceutil.RestartService(service, machine)
			return True
		except:
			return False
	elif action == 'status':
		if win32serviceutil.QueryServiceStatus(service, machine)[1] == 4:
			return True
		else:
			return False
	
	else:
		raise ValueError("Unknown Command")



	    


# ------------------------------------------------
#	Python Siesta
# Siesta is a REST client for python
#
#	Copyright (c) 2008 Rafael Xavier de Souza
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ------------------------------------------------
#
# Modified at Sep 20, 2017 by Hans Roh
#
# ------------------------------------------------

__version__ = "0.5.2"
__author__ = "Sebastian Castillo <castillobuiles@gmail.com>"
__contributors__ = []

import re
import time
import logging
import json
import requests
import base64
from urllib.parse import urlparse, urlencode
from .attrdict import AttrDict
import html2text
import io
import pprint

USER_AGENT = "Python-siesta/%s" % __version__

class Auth(object):
	def encode_params(self, base_url, method, params):
		raise NotImplementedError()

	def make_headers(self):
		pass


class APIKeyAuth(Auth):
	def __init__(self, api_key, auth_header_name="Authorization"):
		self.api_key = api_key
		self.auth_header_name = auth_header_name

	def encode_params(self):
		pass

	def make_headers(self):
		return {self.auth_header_name: self.api_key, }


class BasicAuth(Auth):
	def __init__(self, api_username, api_password, auth_header_name="Authorization"):
		self.api_username = api_username
		self.api_password = api_password
		self.auth_header_name = auth_header_name
		
	def encode_params(self):
		basic_token = base64.encodestring('' + str(self.api_username) + ':' + str(self.api_password))
		basic_token = basic_token.replace('\n', '')
		return basic_token
	
	def make_headers(self):
		token = self.encode_params()
		return {self.auth_header_name: 'Basic ' + token, }

class Response (object):
	def __init__ (self, resource, respnose):
		self.__response = respnose
		self.resource = resource
		self.data = AttrDict ()
		self.set_data (respnose)		
	
	def __getattr__ (self, attr):
		return getattr (self.__response, attr)
	
	def __str__ (self):
		if not isinstance (self.data, str):			
			s = io.StringIO()
			pprint.pprint(self.data, s)
			d = s.getvalue()		
		else:
			d = self.data	
		return "%d %s\n%s" % (self.__response.status_code, self.__response.reason, d)
		
	def set_data (self, resp):
		if not resp.text.strip ():
			self.data = None
		
		else:	
			ct = resp.headers.get ('content-type')
			if ct is None or ct.find ('text/html') == 0:
				h = html2text.HTML2Text()
				h.ignore_links = True
				text = h.handle(resp.text)
				self.data = text
					
			elif ct is None or ct.find ('text/') == 0:
				self.data = resp.text.strip ()
			else:
				data = resp.json ()
				if isinstance (data, dict):
					self.data.update (data)
				else:
					self.data = data
			
		if not str(resp.status_code).startswith("2"):			
			raise AssertionError (self)
		

class Resource(object):
	def __init__(self, uri, api, logger):
		self._api = api
		self._uri = uri
		self._logger = logger
		self._scheme, self._host, self._url = urlparse (self._api.base_url + self._uri) [:3]
		self._headers = {'User-Agent': USER_AGENT, "Accept": 'application/json'}
		
	def __getattr__(self, name):	
		key = self._uri + '/' + name
		return Resource(uri=key, api=self._api, logger = self._logger)		

	def __call__(self, *ids):
		if not ids:
			return self		
		key = self._uri
		for id in ids:
			if key:
				key += '/' + str (id)
			else:
				key += str (id)	
		return Resource(uri=key, api=self._api, logger = self._logger)		
	
	def _request (self, method, data, headers, slash, **kwargs):
		if slash:
			self._url = self._url + "/"
		url = self._url		
		if len(kwargs) > 0:
			url = "%s?%s" % (url, urlencode(kwargs))		
		headers = headers or {"Content-Type": "application/json"}
		return self._continue_request(method, url, data, headers)		
	
	def get(self, headers = None, slash = False, **kwargs):
		return self._request ('GET', None, headers, slash, **kwargs)
	
	def delete(self, headers = None, slash = False, **kwargs):
		return self._request ('DELETE', None, headers, slash, **kwargs)
	
	def options(self, headers = None, slash = False, **kwargs):
		return self._request ('OPTIONS', None, headers, slash, **kwargs)	
				
	def post(self, data, headers = None, slash = False, **kwargs):
		return self._request ('POST', data, headers, slash, **kwargs)
	
	def put(self, data, headers = None, slash = False, **kwargs):
		return self._request ('PUT', data, headers, slash, **kwargs)
	
	def patch(self, data, headers = None, slash = False, **kwargs):
		return self._request ('PATCH', data, headers, slash, **kwargs)

	def _continue_request(self, method, url, data, headers):
		if self._api.auth:
			headers.update(self._api.auth.make_headers())		
		if not 'User-Agent' in headers:
			headers['User-Agent'] = self._headers['User-Agent']
		if not 'Accept' in headers and 'Accept' in self._headers:
			headers['Accept'] = self._headers['Accept']
		
		url = "%s://%s%s" % (self._scheme, self._host, url)		
		if isinstance (data, dict):			
			data = json.dumps (data)
			
		func = getattr (self._api.session, method.lower ())
		if data is not None:
			if not isinstance (data, str):
				raise TypeError ("payload should be str or dic type")
			resp = func (url, data, headers = headers, timeout = self._api.REQ_TIMEOUT, verify = False)	
		else:
			resp = func (url, headers = headers, timeout = self._api.REQ_TIMEOUT, verify = False)
									
		return self._getresponse (resp)
		
	def _getresponse(self, resp):
		if resp.status_code == 202:
			status_url = resp.getheader('content-location')
			if not status_url:
				raise Exception('Empty content-location from server')

			status_uri = urlparse(status_url).path
			resource = Resource(uri=status_uri, api=self._api, logger = self._logger).get()
			retries = 0
			MAX_RETRIES = 3
			resp_status = resource.response.status_code
			
			while resp_status != 303 and retries < MAX_RETRIES:			
				retries += 1
				new_resp.get()
				time.sleep(5)
				
			if retries == MAX_RETRIES:
				raise Exception('Max retries limit reached without success')
			
			location = status.conn.getresponse().getheader('location')
			return Resource(uri=urlparse(location).path, api=self._api, logger = self._logger).get()			
		
		return Response (self, resp)
		

class API(object):
	REQ_TIMEOUT = 60
	def __init__(self, base_url, auth=None, logger = None):
		self.base_url = base_url + '/' if not base_url.endswith('/') else base_url
		self.api_path = urlparse(base_url).path
		self.session = requests.Session ()
		self.auth = auth
		self.logger = logger

	def __getattr__(self, name):
		if name in ('get', 'post', 'put', 'patch', 'delete', 'options'):
			r = Resource(uri='', api=self, logger = self.logger)
			return getattr (r, name)
		return Resource(uri=name, api=self, logger = self.logger)
	
	def __call__(self, id):
		r = Resource(uri='', api=self, logger = self.logger)
		return r (id)



import fcntl, socket, struct

def get_mac (iname):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    info = fcntl.ioctl(s.fileno(), 0x8927,  struct.pack('256s', iname [:15].encode ("utf8")))
    return ':'.join(['%02x' % hex for hex in info[18:24]])

def get_ip ():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(('10.255.255.255', 1))
        IP = s.getsockname()[0]
    except:
        IP = '127.0.0.1'
    finally:
        s.close()
    return IP



import warnings
from functools import wraps

def deprecated (f):
    @wraps(f)
    def wrapper (was, *args, **kwargs):
        warnings.simplefilter ('default')
        warnings.warn (
           "{} will be deprecated".format (f.__name__),
            DeprecationWarning
        )
        return f (was, *args, **kwargs)
    return wrapper


import os

if os.name == "nt":
    class tc_:
        def __getattr__ (self, attr):
            return self.void
        
        def void (self, s):
            return s
    tc = tc_ ()

else:
    class tc:
        WHITE = '\033[97m'
        CYAN = '\033[96m'
        MAGENTA = '\033[95m'
        OKBLUE = '\033[94m'
        WARNING = '\033[93m'
        OKGREEN = '\033[92m'
        FAIL = '\033[91m'
        GREY = '\033[90m'
        
        BOLD = '\033[1m'
        UNDERLINE = '\033[4m'
        ENDC = '\033[0m'
        
        @classmethod
        def _wrap (cls, s, c):
            return "{}{}{}".format (c, s, cls.ENDC)
        
        @classmethod
        def default (cls, s):
            return s
        
        @classmethod
        def critical (cls, s):
            return cls._wrap (s, cls.GREY)

        @classmethod
        def fatal (cls, s):
            return cls._wrap (s, cls.GREY)

        @classmethod
        def info (cls, s):
            return cls._wrap (s, cls.OKGREEN)
        
        @classmethod
        def warn (cls, s):
            return cls._wrap (s, cls.WARNING)
        
        @classmethod
        def expt (cls, s):
            return cls._wrap (s, cls.MAGENTA)
        
        @classmethod
        def fail (cls, s):
            return cls._wrap (s, cls.CYAN)
        
        @classmethod
        def error (cls, s):
            return cls._wrap (s, cls.FAIL)
        
        @classmethod
        def debug (cls, s):
            return cls._wrap (s, cls.WHITE)
                
        @classmethod
        def echo (cls, s):
            return cls._wrap (s, cls.WHITE)
        
        @classmethod
        def primary (cls, s):
            return cls._wrap (s, cls.OKBLUE)
        
        @classmethod
        def secondary (cls, s): 
            return cls._wrap (s, cls.CYAN)
        


from .. import pathtool, logger
import os, signal
import time, sys

class ExitNow (Exception):
    pass

class DaemonClass:
    NAME = "base"
    def __init__ (self, logpath, varpath, consol):        
        self.logpath = logpath
        self.varpath = varpath
        self.consol = consol
        self.last_maintern = 0
        self.flock = None
        self.shutdown_in_progress = False
        
        logpath and pathtool.mkdir (logpath)
        varpath and pathtool.mkdir (varpath)
            
        if not self.consol: # service mode
            sys.stderr = open (os.path.join (varpath, "stderr.log"), "a")
        self.make_logger ()
        self.setup ()
    
    def log (self, msg, type = "info", name = ""):
        self.logger (msg, type, name)
    
    def trace (self, name = ""):
        self.logger.trace (name or self.NAME)    
                
    def make_logger (self):        
        self.logger = logger.multi_logger ()
        if self.consol:
            self.logger.add_logger (logger.screen_logger ())
        if self.logpath:
            self.logger.add_logger (logger.rotate_logger (self.logpath, self.NAME, "weekly"))
            self.log ("{} log path: {}".format (self.NAME, self.logpath), "info")        
        self.log ("{} tmp path: {}".format (self.NAME, self.varpath), "info")
            
    def bind_signal (self, term = None, hup = None):
        if os.name == "nt":
            signal.signal(signal.SIGBREAK, term)
        else:    
            def hUSR1 (signum, frame):    
                self.logger.rotate ()                
            signal.signal(signal.SIGUSR1, hUSR1)
            term and signal.signal(signal.SIGTERM, term)
            term and signal.signal(signal.SIGINT, term)
            hup and signal.signal(signal.SIGHUP, hup)
    
    def start (self):
        self.log ("service %s started" % self.NAME)    
        try:
            self.run ()
        except (ExitNow, KeyboardInterrupt):
            return        
        except:
            self.trace ()
    
    def close (self):
        self.log ("service %s stopped" % self.NAME)
            
    def setup (self):
        pass
    
    def run (self):
        raise NotImplementedError
    
def make_service (service_class, logpath, varpath, consol, *args, **kargs):
    pathtool.mkdir (varpath)
    if logpath:
        pathtool.mkdir (logpath)
    return service_class (logpath, varpath, consol, *args, **kargs)



import os
import wave
import sys
from google.cloud import speech
from google.cloud import translate

"""
Getting CREDENTIAL File:

1. console.cloud.google.com
2. create and download JSON file at "Credentials > Service Account Key" from your project
"""

translate_client = None
speech_client = None

def set_credential (path):
    os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = path  
    
class OverSizeError (Exception):
    pass

def get_translate (text, lang = 'en'):
    global translate_client
    if translate_client is None:
        translate_client = translate.Client()
    translation = translate_client.translate(text, target_language=lang)
    # translation = {'translatedText': 'Hello', 'detectedSourceLanguage': 'ko', 'input': '안녕'}
    return translation['translatedText']

def get_speech (audio_clip_path, lang = 'en-US'):
    global speech_client
    if speech_client is None:
        speech_client = speech.SpeechClient ()
        
    if os.path.getsize(audio_clip_path) > 100000000:
        raise OverSizeError
        
    with wave.open(audio_clip_path,'rb') as temp:
        frequency = temp.getframerate()
    with open(audio_clip_path,'rb') as audio_file:
        content = audio_file.read()

    audio = speech.types.RecognitionAudio(content = content)
    config = speech.types.RecognitionConfig (
        encoding = speech.enums.RecognitionConfig.AudioEncoding.LINEAR16,
        sample_rate_hertz = frequency,
        language_code = lang
    )
    response=speech_client.recognize(config, audio)
    
    if not response.results:
        return ''             
    for result in response.results:
        # Applying Google cloud speech api and recording its result on the database.
        alternatives =result.alternatives
        alternative = alternatives[0].transcript
        if alternative:
            return alternative    
    return ''

    
if __name__ == '__main__':
    print (get_translate ("안녕하세요?"))
    print (get_translate ("Who are you?", "ko"))
    
    print (get_speech ("test-music.wav"))
    print (get_speech ("test-voice.wav"))
    
    